@@ -21,7 +21,7 @@
 
 #include <QPushButton>
 
-#include <qtest.h>
+#include <QTest>
 #include <qtestkeyboard.h>
 #include <qtestmouse.h>
 #include <QComboBox>
