@@ -28,8 +28,6 @@
 
 #include <AkonadiCore/CollectionFetchJob>
 #include <AkonadiCore/ServerManager>
-#include <Akonadi/Contact/ContactGroupExpandJob>
-#include <Akonadi/Contact/ContactGroupSearchJob>
 
 #include <KEmailAddress>
 #include <KLocalizedString>
@@ -91,15 +89,12 @@ public:
             right.first = value.toByteArray();
             Q_EMIT dataChanged(index, index);
             return true;
-            break;
         case PermissionsRole:
             right.second = static_cast<KIMAP::Acl::Rights>(value.toInt());
             Q_EMIT dataChanged(index, index);
             return true;
-            break;
         default:
             return false;
-            break;
         }
 
         return false;
@@ -236,13 +231,16 @@ public:
         if (!dlg.exec()) {
             return;
         }
+        const QString userId = dlg.userId();
+        const QStringList lstAddresses = KEmailAddress::splitAddressList(dlg.userId());
+        for (const QString &addr : lstAddresses) {
+            if (mModel->insertRow(mModel->rowCount())) {
+                const QModelIndex index = mModel->index(mModel->rowCount() - 1, 0);
+                mModel->setData(index, addr, AclModel::UserIdRole);
+                mModel->setData(index, static_cast<int>(dlg.permissions()), AclModel::PermissionsRole);
 
-        if (mModel->insertRow(mModel->rowCount())) {
-            const QModelIndex index = mModel->index(mModel->rowCount() - 1, 0);
-            mModel->setData(index, dlg.userId(), AclModel::UserIdRole);
-            mModel->setData(index, static_cast<int>(dlg.permissions()), AclModel::PermissionsRole);
-
-            mChanged = true;
+                mChanged = true;
+            }
         }
     }
 
@@ -262,10 +260,28 @@ public:
             if (!dlg.exec()) {
                 return;
             }
-
-            mModel->setData(index, dlg.userId(), AclModel::UserIdRole);
-            mModel->setData(index, static_cast<int>(dlg.permissions()), AclModel::PermissionsRole);
-            mChanged = true;
+            const QStringList lstAddresses = KEmailAddress::splitAddressList(dlg.userId());
+            if (lstAddresses.count() == 1) {
+                mModel->setData(index, lstAddresses.at(0), AclModel::UserIdRole);
+                mModel->setData(index, static_cast<int>(dlg.permissions()), AclModel::PermissionsRole);
+                mChanged = true;
+            } else {
+                bool firstElement = true;
+                for (const QString &addr : lstAddresses) {
+                    if (firstElement) {
+                        mModel->setData(index, lstAddresses.at(0), AclModel::UserIdRole);
+                        mModel->setData(index, static_cast<int>(dlg.permissions()), AclModel::PermissionsRole);
+                        firstElement = false;
+                    } else {
+                        if (mModel->insertRow(mModel->rowCount())) {
+                            const QModelIndex index = mModel->index(mModel->rowCount() - 1, 0);
+                            mModel->setData(index, addr, AclModel::UserIdRole);
+                            mModel->setData(index, static_cast<int>(dlg.permissions()), AclModel::PermissionsRole);
+                        }
+                    }
+                }
+                mChanged = true;
+            }
         }
     }
 
@@ -435,42 +451,8 @@ void AclManager::save(bool recursive)
 
     d->mChanged = false;
 
-    QMap<QByteArray, KIMAP::Acl::Rights> newRights;
-
-    const QMap<QByteArray, KIMAP::Acl::Rights> rights = d->mModel->rights();
-    QMap<QByteArray, KIMAP::Acl::Rights>::const_iterator it = rights.cbegin();
-    const QMap<QByteArray, KIMAP::Acl::Rights>::const_iterator itEnd = rights.cend();
-    for (; it != itEnd; ++it) {
-        // we can use job->exec() here, it is not a hot path
-        Akonadi::ContactGroupSearchJob *searchJob = new Akonadi::ContactGroupSearchJob(this);
-        searchJob->setQuery(Akonadi::ContactGroupSearchJob::Name, QString::fromLatin1(it.key()));
-        searchJob->setLimit(1);
-        if (!searchJob->exec()) {
-            continue;
-        }
-
-        if (!searchJob->contactGroups().isEmpty()) {   // it has been a distribution list
-            Akonadi::ContactGroupExpandJob *expandJob
-                = new Akonadi::ContactGroupExpandJob(searchJob->contactGroups().at(0), this);
-            if (expandJob->exec()) {
-                const KContacts::Addressee::List lstContacts = expandJob->contacts();
-                for (const KContacts::Addressee &contact : lstContacts) {
-                    const QByteArray rawEmail
-                        = KEmailAddress::extractEmailAddress(contact.preferredEmail().toUtf8());
-                    if (!rawEmail.isEmpty()) {
-                        newRights[ rawEmail ] = it.value();
-                    }
-                }
-            }
-        } else { // it has been a normal contact
-            const QByteArray rawEmail = KEmailAddress::extractEmailAddress(it.key());
-            if (!rawEmail.isEmpty()) {
-                newRights[ rawEmail ] = it.value();
-            }
-        }
-    }
     PimCommon::AclModifyJob *modifyAclJob = new PimCommon::AclModifyJob;
-    modifyAclJob->setNewRights(newRights);
+    modifyAclJob->setCurrentRight(d->mModel->rights());
     modifyAclJob->setTopLevelCollection(d->mCollection);
     modifyAclJob->setRecursive(recursive);
     modifyAclJob->start();
